----------------------------------------------------------------------------------------
--   File:			InstaWire02.ms
--	 Version:		v02.M042
--	 Started:       09/??/00
--	 Last Modified: 10/27/00
--   Description:	Flexible Instancing of parameters.
--	 Documentation: Visit http://www.gfxcentral.com/bobo/m/test/scripts/instawire
--	 Idea by:		Neil Blevins [Blur Studio]		
--	 Code by:		Borislav Petrov [Boboland]		bobo@email.archlab.tuwien.ac.at
----------------------------------------------------------------------------------------

macroscript InstaWire category:"Bobo_s Tools"
(

on isVisible return ((selection.count == 1)  and (superclassof selection[1] == GeometryClass) and (classof selection[1] != TargetObject))
on isEnabled return ((selection.count == 1)  and (superclassof selection[1] == GeometryClass) and (classof selection[1] != TargetObject))

on Execute do 
(

global instawire_subanims_names_array =#()
global instawire_subanims_tracks_array =#()
global instawire_source_object 

global instawire_float
rollout instawire_roll "InstaWire"
(
edittext instance_object "" align:#center width:150 offset:[0,0]

button inst_all "All" width:48 across:3 align:#left height:18 tooltip:"Check all Tracks for Wire Instancing"
button inst_none "None" width:48  align:#center height:18 tooltip:"Uncheck all Tracks from Wire Instancing"
button inst_invert "Invert" width:48 align:#right height:18 tooltip:"Invert Tags of all Tracks"

listbox param_list items:#() width:150

dropdownlist offset_instance items:#("No offset","+X","+Y","+Z","-X","-Y","-Z") width:70 align:#left across:2
dropdownlist instance_mode items:#("Fan ","Chain") width:70 align:#right
dropdownlist instance_method items:#("Instanced Controller","Wiring")
pickbutton get_new_source  "GET" width:30 height:30 align:#left across:2 tooltip:"Pick a new Object as Source for Wire Instancing..."
button instance_it "MAKE INSTANCE!" width:116 height:30 align:#right tooltip:"Create Wired Instance of the Source!"


fn get_subanim_names source_object =
(
subanims_array = #(source_object)
global instawire_subanims_names_array =#()
global instawire_subanims_tracks_array =#()
i = 0
while i < subanims_array.count do
	(
	i+=1
	for j = 1 to subanims_array[i].numsubs do
		(
		if findstring (subanims_array[i] as string) "Transform" == undefined then
		(
		append subanims_array subanims_array[i][j]
		
		sub_name = (getSubAnimName subanims_array[i] j)
		go_on = true
		if findstring (sub_name as string) "Animation" != undefined then go_on = false
--		if findstring (sub_name as string) "_Wire" != undefined then go_on = false
--		if findstring (sub_name as string) "_" != undefined then go_on = false

		try(if findstring (subanims_array[i].object as string) "_list" != undefined then go_on = false)catch()
		check_class = classof subanims_array[i][j].value
		if check_class != Float and check_class != Quat and check_class != Point3 and check_class != BooleanClass and check_class != Integer then go_on = false 
		str = (subanims_array[i][j] as string)
		if go_on then 
			(
			append instawire_subanims_names_array ("> "+(substring str 9 100))
			append instawire_subanims_tracks_array subanims_array[i][j]
			)
		)	
		)--end j loop
	)--end while
)--end fn

fn update_new_object obj =
(
	instawire_source_object = obj
	get_subanim_names instawire_source_object 
	param_list.items = instawire_subanims_names_array 
	instance_object.text = instawire_source_object.name
	select instawire_source_object
)

on param_list doubleclicked itm do
(
str = instawire_subanims_names_array[itm]
if substring str 1 1 == ">" then 
	str = "  "+(substring str 3 100)
	else
	str = "> "+(substring str 3 100)
instawire_subanims_names_array[itm] = str
param_list.items = instawire_subanims_names_array
)

on inst_invert pressed do
(
for i = 1 to instawire_subanims_names_array.count do
(
str = instawire_subanims_names_array[i]
if substring str 1 1 == ">" then 
	str = "  "+(substring str 3 100)
	else
	str = "> "+(substring str 3 100)
instawire_subanims_names_array[i] = str
)
param_list.items = instawire_subanims_names_array
)

on inst_all pressed do
(
for i = 1 to instawire_subanims_names_array.count do
(
str = instawire_subanims_names_array[i]
str = "> "+(substring str 3 100)
instawire_subanims_names_array[i] = str
)
param_list.items = instawire_subanims_names_array
)

on inst_none pressed do
(
for i = 1 to instawire_subanims_names_array.count do
(
str = instawire_subanims_names_array[i]
str = "  "+(substring str 3 100)
instawire_subanims_names_array[i] = str
)
param_list.items = instawire_subanims_names_array
)


fn animate_track track_name =
(
old_cont = track_name.controller
		if old_cont == undefined then
		(
		old_value = track_name.value 
		try
		(
		at time 11 with animate on track_name.value = false
		)
		catch
		(
		try
			(
			at time 11 with animate on track_name.value = 123
			)
			catch
			(
			try
				(
				at time 11 with animate on track_name.value = [100,200,300]
				)
				catch
				(
				
				)
			)
		)
			deleteKeys track_name.controller #allkeys
			track_name.value = old_value 
			old_cont = track_name.controller
		)
old_cont 		
)


on instance_it pressed do
(
source_object = instawire_source_object
if isValidNode source_object and selection.count == 1 then 
(
target_object = copy source_object
if findstring target_object.name "_IW" == undefined then target_object.name = uniquename (source_object.name + "_IW")
target_object.material = copy source_object.material
if target_object.material != undefined then target_object.material.name = target_object.name

t_o_m = (target_object.max - target_object.min)
case offset_instance.selection of
	(
	1: (offset_delta = [0,0,0])
	2: (offset_delta = [t_o_m.x,0,0])
	3: (offset_delta = [0,t_o_m.y,0])
	4: (offset_delta = [0,0,t_o_m.z])
	5: (offset_delta = [-t_o_m.x,0,0])
	6: (offset_delta = [0,-t_o_m.y,0])
	7: (offset_delta = [0,0,-t_o_m.z])
	)
move target_object offset_delta

subanims_array = #(source_object)
subanims_array2 = #(target_object)
i = 0
while i < subanims_array.count do
	(
	i+=1
	for j = 1 to subanims_array[i].numsubs do
		(
		if findstring (subanims_array[i] as string) "Transform" == undefined then
		(
		append subanims_array subanims_array[i][j]
		append subanims_array2 subanims_array2[i][j]
		
		sub_name = (getSubAnimName subanims_array[i] j)
		go_on = true
		
		test_index = findItem instawire_subanims_tracks_array subanims_array[i][j]
		if test_index > 0 then
		(
		if substring instawire_subanims_names_array[test_index] 1 1 != ">" then go_on = false
		)
		else
		(
		go_on = false
		)
		
		if findstring (sub_name as string) "_Wire" != undefined then go_on = false
		
		try(if findstring (subanims_array[i].object as string) "_list" != undefined then go_on = false)catch()
		check_class = classof subanims_array[i][j].value
		if check_class != Float and check_class != Quat and check_class != Point3 and check_class != BooleanClass and check_class != Integer then go_on = false 
		if go_on then 
			(
			try 
				(
				case instance_method.selection of
					(
					2: paramwire.connect2way subanims_array[i][j] subanims_array2[i][j] (sub_name as string) (sub_name as string)
					1: (
						if subanims_array[i][j].controller == undefined then
						(
						animate_track subanims_array[i][j]
						)
						subanims_array2[i][j].controller = subanims_array[i][j].controller
						)
					)
				)
				catch
				(
				format "Failed to Instance %\n" subanims_array[i][j]
				)
			)
		)	
		)--end j loop
	)--end while
if instance_mode.selection == 2 then 
	(
	instawire_source_object = target_object
	get_subanim_names instawire_source_object 
	param_list.items = instawire_subanims_names_array 
	instance_object.text = instawire_source_object.name
	select instawire_source_object
	)	
)
else	
(
messagebox "Selection is Invalid!" title:"InstaWire"
)
)--end on	

on get_new_source picked obj do
(
update_new_object obj
)

)--end roll

try(closerolloutfloater instawire_float)catch()
instawire_float = newrolloutfloater "InstaWire v0.2" 200 346 100 100
addrollout instawire_roll instawire_float
instawire_roll.update_new_object $
)





)--end script